#!/usr/bin/php -q
<?php
/* Created by Aimon Bustardo @ Morph Labs */
/* Updated by Richard Ablewhite */

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0]) || isset($_SERVER['REQUEST_METHOD'])  || isset($_SERVER['REMOTE_ADDR'])) {
	die("<br><strong>This script is only meant to run at the command line.</strong>");
} 
 
/* We are not talking to the browser */
$no_http_headers = true;

include(dirname(__FILE__)."/../site/include/global.php");
include_once($config["base_path"]."/lib/api_automation_tools.php");
include_once($config["base_path"]."/lib/utility.php");
include_once($config["base_path"]."/lib/api_data_source.php");
include_once($config["base_path"]."/lib/api_graph.php");
include_once($config["base_path"]."/lib/snmp.php");
include_once($config["base_path"]."/lib/data_query.php");
include_once($config["base_path"]."/lib/api_device.php");


/* process calling arguments*/
$parms = $_SERVER["argv"];
array_shift($parms);

if (!sizeof($parms)) {
	display_help();
	exit(0);
}
/* set up defaults*/
$hosts          = getHosts();
$quietMode      = FALSE;

foreach($parms as $parameter) {
	@list($arg, $value) = @explode("=", $parameter);
		
	switch ($arg) {
		
		case "--hostname":
			
			$dids = array();
			
			foreach($hosts as $host)
				if($host['hostname'] == $value)
					$dids[] = $host['id'] ;
			
			if(sizeof($dids)<1)
			{
				echo "No host found with that hostname.\n" ;
				exit(1);
			}
			
			deleteDevices($dids);
			exit(0);
			break;
		
		case "--device-id":
			if(!sizeof($value)){
				echo "ERROR: No device IDs supplied.\n";
				display_help();
				exit(1);
			}
			
			$dids = @explode(",", $value);
			deleteDevices($dids);
			
			exit(0);
			break;
		case "--list-devices":
			displayHosts($hosts, $quietMode);
			exit(0);
			break;
		default:
			echo "ERROR: Missing or extraneous aruments.\n\n";
			display_help();
			exit(1);
	}

}

function deleteDevices($dids) {
	global $hosts ;
	
	$a=1;
	foreach($dids as $did){
		if(!is_numeric($did)){
			echo "ERROR: Number $a in the supplied device ID is not a number.\n\n";
			display_help();
			exit(1);
		}
		if(!array_key_exists($did, $hosts)){
			echo "ERROR: Device ID $did does not exist. Aborting.\n\n";
			display_help();
			exit(1);
		}
		$a++;
	}
	
	$data_sources_to_act_on = array();
	$graphs_to_act_on       = array();
	$devices_to_act_on      = array();

	for ($i=0; $i<count($dids); $i++) {
		$data_sources = db_fetch_assoc("select
			data_local.id as local_data_id
			from data_local
			where " . array_to_sql_or($dids, "data_local.host_id"));

		if (sizeof($data_sources) > 0) {
		foreach ($data_sources as $data_source) {
			$data_sources_to_act_on[] = $data_source["local_data_id"];
		}
		}

		$graphs = db_fetch_assoc("select
			graph_local.id as local_graph_id
			from graph_local
			where " . array_to_sql_or($dids, "graph_local.host_id"));

		if (sizeof($graphs) > 0) {
			foreach ($graphs as $graph) {
				$graphs_to_act_on[] = $graph["local_graph_id"];
			}
		}

		$devices_to_act_on[] = $dids[$i];
	}

	/* All keys are valid and exist, processing removes*/

	api_data_source_remove_multi($data_sources_to_act_on);

	api_graph_remove_multi($graphs_to_act_on);

	api_plugin_hook_function('graphs_remove', $graphs_to_act_on);

	api_device_remove_multi($devices_to_act_on);

	api_plugin_hook_function('device_remove', $devices_to_act_on);
}

function display_help() {
	echo "Remove Device Script 0.2\n\n";
	echo "A command line utility to remove a device or multiple devices from Cacti\n\n";
	echo "Removes all associated graphs and data sources.\n\n";
	echo "Remove device usage by device ID: remove_device.php --device-id=[device ids] \n";
	echo "Remove device usage by hostname ID: remove_device.php ----hostname=[hostname] \n\n";
	echo "Required:\n";
	echo "    --device-id    comma separated list of host/device IDs.\n";
	echo "Or:\n";
	echo "    --hostname     device hostname.\n\n";
	echo "List devices usage: remove_device.php --list-devices\n\n";
	echo "List Options:\n";
	echo "    --list-devices    Lists all devices\n\n";
}



